@@ -301,7 +301,7 @@ pub fn solve(input: &mut Input) -> Result<i32, String> {
 
 #[test]
 fn tests() {
-    use crate::{test_part_one, test_part_one_error, test_part_two};
+    use crate::input::{test_part_one, test_part_one_error, test_part_two};
 
     test_part_one_error!("GE" => "No line in input");
     test_part_one_error!("GE\n" => "Map does not have a wall at edges");
