@@ -1,17 +1,15 @@
 use crate::input::Input;
 use std::collections::hash_map::Entry;
 use std::collections::{HashMap, HashSet};
-use std::ops::RangeInclusive;
 
 type CoordinateComponent = i8;
 
-#[derive(Copy, Clone, Hash, Eq, PartialEq)]
-struct Coordinate {
-    x: CoordinateComponent,
-    y: CoordinateComponent,
-    z: CoordinateComponent,
-    w: CoordinateComponent,
-}
+type Coordinate = (
+    CoordinateComponent,
+    CoordinateComponent,
+    CoordinateComponent,
+    CoordinateComponent,
+);
 
 struct Grid {
     occupied_coordinates: HashSet<Coordinate>,
@@ -28,12 +26,12 @@ impl Grid {
             }
             for (col, b) in line.bytes().enumerate() {
                 if b == b'#' {
-                    occupied_coordinates.insert(Coordinate {
-                        x: col as CoordinateComponent,
-                        y: row as CoordinateComponent,
-                        z: 0,
-                        w: 0,
-                    });
+                    occupied_coordinates.insert((
+                        col as CoordinateComponent,
+                        row as CoordinateComponent,
+                        0,
+                        0,
+                    ));
                 }
             }
         }
@@ -44,26 +42,22 @@ impl Grid {
         })
     }
 
-    fn cycle(&mut self, w_range: &RangeInclusive<CoordinateComponent>) {
+    fn cycle(&mut self, w_range: CoordinateComponent) {
         self.active_neighbors_count.clear();
 
         for coordinate in self.occupied_coordinates.iter() {
             for dx in -1..=1 {
                 for dy in -1..=1 {
                     for dz in -1..=1 {
-                        for dw in w_range.clone() {
-                            let coordinate = Coordinate {
-                                x: coordinate.x + dx,
-                                y: coordinate.y + dy,
-                                z: coordinate.z + dz,
-                                w: coordinate.w + dw,
-                            };
+                        for dw in -w_range..=w_range {
+                            let coordinate = (
+                                coordinate.0 + dx,
+                                coordinate.1 + dy,
+                                coordinate.2 + dz,
+                                coordinate.3 + dw,
+                            );
 
-                            let value_to_add = if dx == 0 && dy == 0 && dz == 0 && dw == 0 {
-                                0
-                            } else {
-                                1
-                            };
+                            let value_to_add = (dx | dy | dz | dw).abs() as u8;
 
                             match self.active_neighbors_count.entry(coordinate) {
                                 Entry::Occupied(mut entry) => {
@@ -81,15 +75,17 @@ impl Grid {
 
         for (&coordinate, &active_neighbors) in self.active_neighbors_count.iter() {
             let is_active = self.occupied_coordinates.contains(&coordinate);
+
             let will_be_active = if is_active {
-                // - If a cube is active and exactly 2 or 3 of its neighbors are also active,
-                // the cube remains active. Otherwise, the cube becomes inactive.
+                // "If a cube is active and exactly 2 or 3 of its neighbors are also active,
+                // the cube remains active. Otherwise, the cube becomes inactive."
                 active_neighbors == 2 || active_neighbors == 3
             } else {
-                // - If a cube is inactive but exactly 3 of its neighbors are active, the cube
-                // becomes active. Otherwise, the cube remains inactive.
+                // "If a cube is inactive but exactly 3 of its neighbors are active, the cube
+                // becomes active. Otherwise, the cube remains inactive."
                 active_neighbors == 3
             };
+
             if is_active && !will_be_active {
                 self.occupied_coordinates.remove(&coordinate);
             } else if !is_active && will_be_active {
@@ -101,10 +97,10 @@ impl Grid {
 
 pub fn solve(input: &mut Input) -> Result<u64, String> {
     let mut grid = Grid::parse(input.text)?;
-    let w_range = input.part_values(0..=0, -1..=1);
+    let w_range = input.part_values(0, 1);
 
     for _ in 0..6 {
-        grid.cycle(&w_range);
+        grid.cycle(w_range);
     }
 
     Ok(grid.occupied_coordinates.len() as u64)
